# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv, orm
from osv.osv import osv_pool, object_proxy
from osv.orm import orm_template
from tools.translate import _
from lxml import etree
from datetime import *
import ir
import pooler
import time
import tools
import logging
from tools.safe_eval import safe_eval as eval
import logging

class purchase_order(osv.osv):
    _name = 'purchase.order'
    _inherit = 'purchase.order'
    _trace = True

purchase_order()

class purchase_order_line(osv.osv):
    _name = 'purchase.order.line'
    _inherit = 'purchase.order.line'
    _trace = True

purchase_order_line()

class sale_order(osv.osv):
    _name = 'sale.order'
    _inherit = 'sale.order'
    _trace = True

sale_order()

class sale_order_line(osv.osv):
    _name = 'sale.order.line'
    _inherit = 'sale.order.line'
    _trace = True

sale_order_line()

class stock_picking(osv.osv):
    _name = 'stock.picking'
    _inherit = 'stock.picking'
    _trace = True

stock_picking()

class stock_move(osv.osv):
    _name = 'stock.move'
    _inherit = 'stock.move'
    _trace = True

    # [utp-360]: I rename the 'date' to 'Actual Receipt Date' because before it was 'Date'
    _columns = {
        'date': fields.datetime('Actual Receipt Date', required=True, select=True, help="Move date: scheduled date until move is done, then date of actual move processing", readonly=True),
    }

stock_move()

class account_invoice(osv.osv):
    _name = 'account.invoice'
    _inherit = 'account.invoice'
    _trace = True

account_invoice()

class account_invoice_line(osv.osv):
    _name = 'account.invoice.line'
    _inherit = 'account.invoice.line'
    _trace = True

account_invoice_line()

class account_bank_statement(osv.osv):
    _name = 'account.bank.statement'
    _inherit = 'account.bank.statement'
    _trace = True

account_bank_statement()

class account_bank_statement_line(osv.osv):
    _name = 'account.bank.statement.line'
    _inherit = 'account.bank.statement.line'
    _trace = True

    def _get_partner_type2(self, cr, uid, ids, field_name=None, arg=None, context=None):
        """
        Get "Third Parties" audittrail version
        (audittrail does not process field function reference for now)
        """
        res = {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        for st_line in self.browse(cr, uid, ids, context=context):
            if st_line.employee_id:
                res[st_line.id] = st_line.employee_id.name
            elif st_line.transfer_journal_id:
                res[st_line.id] = st_line.transfer_journal_id.name
            elif st_line.partner_id:
                res[st_line.id] = st_line.partner_id.name
            else:
                res[st_line.id] = False
        return res

    _columns = {
        'partner_type2': fields.function(_get_partner_type2, method=True, string="Third Parties"),
    }

    _defaults = {
        'name': lambda *a: '',
    }

account_bank_statement_line()

class account_analytic_account(osv.osv):
    _name = 'account.analytic.account'
    _inherit = 'account.analytic.account'
    _trace = True

account_analytic_account()

class account_period(osv.osv):
    _name = 'account.period'
    _inherit = 'account.period'
    _trace = True

account_period()


class ir_module(osv.osv):
    _inherit = 'ir.module.module'

    def update_translations(self, cr, uid, ids, filter_lang=None, context=None):
        '''
        Override the lang install to apply the translation on Track changes ir.actions
        '''
        res = super(ir_module, self).update_translations(cr, uid, ids, filter_lang=None, context=context)

        msf_profile_id = self.search(cr, uid, [('name', '=', 'msf_profile')], context=context)
        
        if not msf_profile_id or msf_profile_id[0] not in ids:
            return res

        tr_obj = self.pool.get('ir.translation')
        act_obj = self.pool.get('ir.actions.act_window')
        language_obj = self.browse(cr, uid, ids)[0]
        src = 'Track changes'
        if not filter_lang:
            pool = pooler.get_pool(cr.dbname)
            lang_obj = pool.get('res.lang')
            lang_ids = lang_obj.search(cr, uid, [('translatable', '=', True)])
            filter_lang = [lang.code for lang in lang_obj.browse(cr, uid, lang_ids)]
        elif not isinstance(filter_lang, (list, tuple)):
            filter_lang = [filter_lang]

        for lang in filter_lang:
            trans_ids = tr_obj.search(cr, uid, [('lang', '=', lang),
                                                ('xml_id', '=', 'action_audittrail_view_log'),
                                                ('module', '=', 'msf_audittrail')], context=context)
            if trans_ids:
                logger = logging.getLogger('i18n')
                logger.info('module msf_profile: loading translation for \'Track changes\' ir.actions.act_window for language %s', lang)
                trans = tr_obj.browse(cr, uid, trans_ids[0], context=context).value
                # Search all actions to rename
                act_ids = act_obj.search(cr, uid, [('name', '=', src)], context=context)
                for act in act_ids:
                    exist = tr_obj.search(cr, uid, [('lang', '=', lang), 
                                                    ('type', '=', 'model'), 
                                                    ('src', '=', src), 
                                                    ('name', '=', 'ir.actions.act_window,name'), 
                                                    ('value', '=', trans), 
                                                    ('res_id', '=', act)], context=context)
                    if not exist:
                        tr_obj.create(cr, uid, {'lang': lang,
                                                'src': src,
                                                'name': 'ir.actions.act_window,name',
                                                'type': 'model',
                                                'value': trans,
                                                'res_id': act}, context=context)

        return res

ir_module()


class audittrail_log_sequence(osv.osv):
    _name = 'audittrail.log.sequence'
    _rec_name = 'model'
    _columns = {
        'model': fields.char(size=64, string='Model'),
        'res_id': fields.integer(string='Res Id'),
        'sequence': fields.many2one('ir.sequence', 'Logs Sequence', required=True, ondelete='cascade'),
    }

audittrail_log_sequence()

class audittrail_rule(osv.osv):
    """
    For Auddittrail Rule
    """
    _name = 'audittrail.rule'
    _description = "Audittrail Rule"
    _columns = {
        "name": fields.char("Rule Name", size=32, required=True),
        "object_id": fields.many2one('ir.model', 'Object', required=True, help="Select object for which you want to generate log."),
        "log_read": fields.boolean("Log Reads", help="Select this if you want to keep track of read/open on any record of the object of this rule"),
        "log_write": fields.boolean("Log Writes", help="Select this if you want to keep track of modification on any record of the object of this rule"),
        "log_unlink": fields.boolean("Log Deletes", help="Select this if you want to keep track of deletion on any record of the object of this rule"),
        "log_create": fields.boolean("Log Creates",help="Select this if you want to keep track of creation on any record of the object of this rule"),
        "log_action": fields.boolean("Log Action",help="Select this if you want to keep track of actions on the object of this rule"),
        "log_workflow": fields.boolean("Log Workflow",help="Select this if you want to keep track of workflow on any record of the object of this rule"),
        "domain_filter": fields.char(size=128, string="Domain", help="Python expression !"),
        "state": fields.selection((("draft", "Draft"),
                                   ("subscribed", "Subscribed")),
                                   "State", required=True),
        "action_id": fields.many2one('ir.actions.act_window', "Action ID"),
        "field_ids": fields.many2many('ir.model.fields', 'audit_rule_field_rel', 'rule_id', 'field_id', string='Fields'),
        "parent_field_id": fields.many2one('ir.model.fields', string='Parent fields'),
        "name_get_field_id": fields.many2one('ir.model.fields', string='Displayed field value'),
    }

    _defaults = {
        'state': lambda *a: 'draft',
        'log_create': lambda *a: 1,
        'log_unlink': lambda *a: 1,
        'log_write': lambda *a: 1,
        'domain_filter': [],
    }

# we replace the sql_constraint below by a Python constraint which checks that there is one type of rule per type.
#    _sql_constraints = [
#        ('model_uniq', 'unique (object_id)', """There is a rule defined on this object\n You can not define other on the same!""")
#    ]

    _sql_constraints = [
        ('rule_name_uniq', 'unique(name)', """The AuditTrail rule name must be unique!""")
    ]

    def _check_domain_filter(self, cr, uid, ids, context=None):
        """
        Check that if you select cross docking, you do not have an other location than cross docking
        """
        if isinstance(ids, (int, long)):
            ids = [ids]
        if context is None:
            context = {}
        
        for rule in self.browse(cr, uid, ids, context=context):
            domain = eval(rule.domain_filter)
            for d in tuple(domain):
                if len(d[0].split('.')) > 2:
                    return False
        
        return True
    
    _constraints = [
        (_check_domain_filter, 'The domain shouldn\'t contain a right element in condition with more than 2 elements.', ['domain_filter']),
    ]
    __functions = {}

    def subscribe(self, cr, uid, ids, *args):
        """
        Subscribe Rule for auditing changes on object and apply shortcut for logs on that object.
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param ids: List of Auddittrail Rule’s IDs.
        @return: True
        """
        if isinstance(ids, (int, long)):
            ids = [ids]
        
        obj_action = self.pool.get('ir.actions.act_window')
        obj_model = self.pool.get('ir.model.data')
        #start Loop
        for thisrule in self.browse(cr, uid, ids):
            obj = self.pool.get(thisrule.object_id.model)
            if not obj:
                raise osv.except_osv(
                        _('WARNING: audittrail is not part of the pool'),
                        _('Change audittrail depends -- Setting rule as DRAFT'))
                self.write(cr, uid, [thisrule.id], {"state": "draft"})
            search_view_id = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'msf_audittrail', 'view_audittrail_log_line_search')
            val = {
                 "name": _('Track changes'),
                 "res_model": 'audittrail.log.line',
                 "src_model": thisrule.object_id.model,
                 "search_view_id": search_view_id and search_view_id[1] or False,
                 "domain": "[('object_id','=', " + str(thisrule.object_id.id) + "), ('res_id', '=', active_id)]"
            }
            if thisrule.object_id.model == 'account.bank.statement.line':
                # for register line we allow to select many lines in track changes view
                val['domain'] = "[('object_id','=', " + str(thisrule.object_id.id) + "), ('res_id', 'in', active_ids)]"

            action_id = obj_action.create(cr, uid, val)
            self.write(cr, uid, [thisrule.id], {"state": "subscribed", "action_id": action_id})
            keyword = 'client_action_relate'
            value = 'ir.actions.act_window,' + str(action_id)
            obj_model.ir_set(cr, uid, 'action', keyword, 'View_log_' + thisrule.object_id.model, [thisrule.object_id.model], value, replace=True, isobject=True, xml_id=False)
            #End Loop
        
        # Check if an export model already exist for audittrail.rule
        export_ids = self.pool.get('ir.exports').search(cr, uid, [('name', '=', 'Log Lines'), ('resource', '=', 'audittrail.log.line')])
        if not export_ids:
            export_id = self.pool.get('ir.exports').create(cr, uid, {'name': 'Log Lines',
                                                                     'resource': 'audittrail.log.line'})
            fields = ['log', 'timestamp', 'sub_obj_name', 'method', 'field_description', 'old_value', 'new_value', 'user_id']
            for f in fields:
                self.pool.get('ir.exports.line').create(cr, uid, {'name': f, 'export_id': export_id}) 

        return True

    def unsubscribe(self, cr, uid, ids, *args):
        """
        Unsubscribe Auditing Rule on object
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param ids: List of Auddittrail Rule’s IDs.
        @return: True
        """
        obj_action = self.pool.get('ir.actions.act_window')
        val_obj = self.pool.get('ir.values')
        value=''
        #start Loop
        for thisrule in self.browse(cr, uid, ids):
            if thisrule.id in self.__functions:
                for function in self.__functions[thisrule.id]:
                    setattr(function[0], function[1], function[2])
            w_id = obj_action.search(cr, uid, [('name', '=', 'View Log'), ('res_model', '=', 'audittrail.log.line'), ('src_model', '=', thisrule.object_id.model)])
            if w_id:
                obj_action.unlink(cr, uid, w_id)
                value = "ir.actions.act_window" + ',' + str(w_id[0])
            val_id = val_obj.search(cr, uid, [('model', '=', thisrule.object_id.model), ('value', '=', value)])
            if val_id:
                ir.ir_del(cr, uid, val_id[0])
            self.write(cr, uid, [thisrule.id], {"state": "draft"})
        #End Loop
        
        return True

audittrail_rule()


class audittrail_log_line(osv.osv):
    """
    Audittrail Log Line.
    """
    _name = 'audittrail.log.line'
    _description = "Log Line"
    _order = 'timestamp asc'

    def _get_name_line(self, cr, uid, ids, field_name, args, context=None):
        '''
        Return the value of the field set in the rule
        '''
        res = {}

        for line in self.browse(cr, uid, ids, context=context):
            if not line.rule_id or not line.fct_res_id or not line.fct_object_id:
                res[line.id] = False
            else:
                field = line.rule_id.name_get_field_id.name
                res_id = line.fct_res_id
                object_id = self.pool.get(line.fct_object_id.model)
                try:
                    res[line.id] = object_id.read(cr, uid, res_id, [field], context=context)[field]
                except TypeError:
                    res[line.id] = False

        return res

    ####
    # TODO : To validate
    ####
    def _search_name_line(self, cr, uid, obj, name, args, context=None):
        '''
        Returns all lines corresponding to the args
        '''
        ids = []

        if not context:
            return []

        for arg in args:
            if not arg[2]:
                return []
            if arg[0] == 'sub_obj_name' and arg[1] == 'ilike' and arg[2]:
                line_ids = self.browse(cr, uid, context.get('active_ids'), context=context)
                for line in line_ids:
                    if line.rule_id and line.fct_res_id and line.fct_object_id:
                        field = line.rule_id.name_get_field_id.name
                        res_id = line.fct_res_id
                        object_id = self.pool.get(line.fct_object_id.model)
                        if str(object_id.read(cr, uid, res_id, [field], context=context)[field]) == arg[2]:
                            ids.append(line.id)
                
                return [('id', 'in', ids)]

        return []
    
    def _get_values(self, cr, uid, ids, field_name, arg, context=None):
        '''
        Return the value of the field according to his type
        '''
        res = {}
        
        for line in self.browse(cr, uid, ids, context=context):
            res[line.id] = {'old_value_fct': False, 'new_value_fct': False}
            if not line.old_value_text:
                res[line.id]['old_value_fct'] = get_value_text(self, cr, uid, line.field_id.id, False, line.old_value, line.fct_object_id or line.object_id, context=context)
            else:
                res[line.id]['old_value_fct'] = line.old_value_text
            if not line.new_value_text:
                res[line.id]['new_value_fct'] = get_value_text(self, cr, uid, line.field_id.id, False, line.new_value, line.fct_object_id or line.object_id, context=context)
            else:
                res[line.id]['new_value_fct'] = line.new_value_text
                
            if not line.old_value_text and not line.new_value_text:
                self.write(cr, uid, [line.id], {'old_value_text': res[line.id]['old_value_fct'], 'new_value_text': res[line.id]['new_value_fct']})
            elif not line.old_value_text:
                self.write(cr, uid, [line.id], {'old_value_text': res[line.id]['old_value_fct'],})
            elif not line.new_value_text:
                self.write(cr, uid, [line.id], {'new_value_text': res[line.id]['new_value_fct'],})
        
        return res

    def _get_field_name(self, cr, uid, ids, field_name, arg, context=None):
        '''
        Return the name of the field in the user language
        '''
        tr_obj = self.pool.get('ir.translation')

        res = {}
        lang = self.pool.get('res.users').browse(cr, uid, uid, context=context).context_lang

        for line in self.browse(cr, uid, ids, context=context):
            res[line.id] = False

            # Translation of field name
            if line.field_id:
                field_name = '%s,%s' % (line.object_id.model, line.field_id.name)
                tr_ids = tr_obj.search(cr, uid, [('name', '=', field_name),
                                                 ('lang', '=', lang),
                                                 ('type', '=', 'field'),
                                                 ('src', '=', line.field_id.field_description)], context=context)
                if tr_ids:
                    res[line.id] = tr_obj.browse(cr, uid, tr_ids[0], context=context).value

            # Translation of one2many object if any
            if not res[line.id] and line.fct_object_id:
                field_name = '%s,%s' % (line.fct_object_id.model, line.field_id.name)
                tr_ids = tr_obj.search(cr, uid, [('name', '=', field_name),
                                                 ('lang', '=', lang),
                                                 ('type', '=', 'field'),
                                                 ('src', '=', line.field_id.field_description)], context=context)
                if tr_ids:
                    res[line.id] = tr_obj.browse(cr, uid, tr_ids[0], context=context).value

            # Translation of main object
            if not res[line.id] and (line.object_id or line.fct_object_id):
                tr_ids = tr_obj.search(cr, uid, [('name', '=', 'ir.model,name'),
                                                 ('lang', '=', lang),
                                                 ('type', '=', 'model'),
                                                 ('src', '=', line.name)], context=context)
                if tr_ids:
                    res[line.id] = tr_obj.browse(cr, uid, tr_ids[0], context=context).value

            # No translation
            if not res[line.id]:
                res[line.id] = line.field_description

        return res

    def _src_field_name(self, cr, uid, obj, name, args, context=None):
        '''
        Search field description with the user lang
        '''
        tr_obj = self.pool.get('ir.translation')

        res = []
        lang = self.pool.get('res.users').browse(cr, uid, uid, context=context).context_lang

        for arg in args:
            if arg[0] == 'trans_field_description':
                tr_fields = tr_obj.search(cr, uid, [('lang', '=', lang), 
                                                    ('type', 'in', ['field', 'model']),
                                                    ('value', arg[1], arg[2])], context=context)

                field_names = []
                for f in tr_obj.browse(cr, uid, tr_fields, context=context):
                    field_names.append(f.src)

                res = [('field_description', 'in', field_names)]

        return res


    _columns = {
          'name': fields.char(size=256, string='Description', required=True),
          'object_id': fields.many2one('ir.model', string='Object'),
          'user_id': fields.many2one('res.users', string='User'),
          'method': fields.selection([('create', 'Creation'), ('write', 'Modification'), ('unlink', 'Deletion')], string='Method'),
          'timestamp': fields.datetime(string='Date'),
          'res_id': fields.integer(string='Resource Id'),
          'field_id': fields.many2one('ir.model.fields', 'Fields'),
          'log': fields.integer("Log ID"),
          'old_value_fct': fields.function(_get_values, method=True, string='Old Value Fct', type='char', store=False, multi='values'),
          'new_value_fct': fields.function(_get_values, method=True, string='New Value Fct', type='char', store=False, multi='values'),
          'old_value_text': fields.char(size=256, string='Old Value'),
          'new_value_text': fields.char(size=256, string='New Value'),
          'old_value': fields.text("Old Value"),
          'new_value': fields.text("New Value"),
          'field_description': fields.char('Field Description', size=64),
          'trans_field_description': fields.function(_get_field_name, fnct_search=_src_field_name, method=True, type='char', size=64, string='Field Description', store=False),
          'sub_obj_name': fields.char(size=64, string='Order line'),
#          'sub_obj_name': fields.function(fnct=_get_name_line, fnct_search=_search_name_line, method=True, type='char', string='Order line', store=False),
          # These 3 fields allows the computation of the name of the subobject (sub_obj_name)
          'rule_id': fields.many2one('audittrail.rule', string='Rule'),
          'fct_res_id': fields.integer(string='Res. Id'),
          'fct_object_id': fields.many2one('ir.model', string='Fct. Object'),
        }

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        '''
        Display the name of the resource on the tree view
        '''
        res = super(osv.osv, self).fields_view_get(cr, uid, view_id, view_type, context=context, toolbar=toolbar, submenu=submenu)
        # TODO: Waiting OEB-86 
#        if view_type == 'tree' and context.get('active_ids') and context.get('active_model'):
#            element_name = self.pool.get(context.get('active_model')).name_get(cr, uid, context.get('active_ids'), context=context)[0][1]
#            xml_view = etree.fromstring(res['arch'])
#            for element in xml_view.iter("tree"):
#                element.set('string', element_name)
#            res['arch'] = etree.tostring(xml_view)
        return res

    def _get_report_name(self, cr, uid, ids, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]

        self_info = self.browse(cr, uid, ids[0], context)
        name = self_info.object_id.name or ''
        if self_info.res_id and self_info.object_id.model:
            obj = self.pool.get(self_info.object_id.model)
            if obj:
                name_get = obj.name_get(cr, uid, [self_info.res_id])
                if name_get and name_get[0]:
                    name = name_get[0][1].replace('/','_')
        return "LL_%s_%s" % (name, time.strftime('%Y%m%d'))

audittrail_log_line()


def get_value_text(self, cr, uid, field_id, field_name, values, model, context=None):
    """
    Gets textual values for the fields
    e.g.: For field of type many2one it gives its name value instead of id

    @param cr: the current row, from the database cursor,
    @param uid: the current user’s ID for security checks,
    @param field_name: List of fields for text values
    @param values: Values for field to be converted into textual values
    @return: values: List of textual values for given fields
    """
    if not context:
        context = {}
    if field_name in('__last_update','id'):
        return values
    pool = pooler.get_pool(cr.dbname)
    field_pool = pool.get('ir.model.fields')
    model_pool = pool.get('ir.model')
    if not field_id:
        obj_pool = pool.get(model.model)
        # If the field is not in the current object,
        # search if it's in an inherited object
        if obj_pool._inherits:
            inherits_ids = model_pool.search(cr, uid, [('model', '=', obj_pool._inherits.keys()[0])])
            field_ids = field_pool.search(cr, uid, [('name', '=', field_name), ('model_id', 'in', (model.id, inherits_ids[0]))])
        else:
            field_ids = field_pool.search(cr, uid, [('name', '=', field_name), ('model_id', '=', model.id)])
        field_id = field_ids and field_ids[0] or False

    if field_id:
        field = field_pool.read(cr, uid, field_id)
        relation_model = field['relation']
        relation_model_pool = relation_model and pool.get(relation_model) or False

        if field['ttype'] == 'many2one':
            res = False
            if values and values != '()':
                values = values[1:-1].split(',')
                if len(values) and values[0] != '' and relation_model_pool:
                    # int() failed if value '167L'
                    relation_model_object = relation_model_pool.read(cr, uid, long(values[0]), [relation_model_pool._rec_name])
                    res = relation_model_object[relation_model_pool._rec_name]
            return res

        elif field['ttype'] in ('many2many','one2many'):
            res = []
            if values and values != '[]':
                values = values[1:-1].split(',')
                for v in values:
                    relation_model_object = relation_model_pool.read(cr, uid, int(v), [relation_model_pool._rec_name])
                    res.append(relation_model_object[relation_model_pool._rec_name])
            return res
        elif field['ttype'] == 'date':
            res = False
            if values:
                date_format = self.pool.get('date.tools').get_date_format(cr, uid, context=context)
                res = datetime.strptime(values, '%Y-%m-%d')
                res = datetime.strftime(res, date_format)
            return res
        elif field['ttype'] == 'datetime':
            res = False
            if values:
                # Display only the date on log line (Comment the next line and uncomment the next one if you want display the time)
                date_format = self.pool.get('date.tools').get_date_format(cr, uid, context=context)
                #date_format = self.pool.get('date.tools').get_datetime_format(cr, uid, context=context)
                try:
                    res = datetime.strptime(values, '%Y-%m-%d %H:%M:%S')
                except ValueError:
                    res = datetime.strptime(values, '%Y-%m-%d %H:%M:%S.%f')
                finally:
                    res = datetime.strftime(res, date_format)
            return res
        elif field['ttype'] == 'selection':
            res = False
            if values:
                fct_object = model_pool.browse(cr, uid, model.id, context=context).model
                sel = self.pool.get(fct_object).fields_get(cr, uid, [field['name']])
                res = dict(sel[field['name']]['selection']).get(values)
                name = '%s,%s' % (fct_object, field['name'])
                # Search translation
                res_tr_ids = self.pool.get('ir.translation').search(cr, uid, [('type', '=', 'selection'), ('name', '=', name),('src', 'in', [values])])
                if res_tr_ids:
                    res = self.pool.get('ir.translation').read(cr, uid, res_tr_ids, ['value'])[0]['value']
            return res

    return values

def create_log_line(self, cr, uid, model, lines=[]):
    """
    Creates lines for changed fields with its old and new values

    @param cr: the current row, from the database cursor,
    @param uid: the current user’s ID for security checks,
    @param model: Object who's values are being changed
    @param lines: List of values for line is to be created
    """
    pool = pooler.get_pool(cr.dbname)
    obj_pool = pool.get(model.model)
    model_pool = pool.get('ir.model')
    field_pool = pool.get('ir.model.fields')
    log_line_pool = pool.get('audittrail.log.line')
    #start Loop
    for line in lines:
        dict_of_values = {}
        if line['name'] in('__last_update','id'):
            continue
        if obj_pool._inherits:
            inherits_ids = model_pool.search(cr, uid, [('model', '=', obj_pool._inherits.keys()[0])])
            field_ids = field_pool.search(cr, uid, [('name', '=', line['name']), ('model_id', 'in', (model.id, inherits_ids[0]))])
        else:
            field_ids = field_pool.search(cr, uid, [('name', '=', line['name']), ('model_id', '=', model.id)])
        field_id = field_ids and field_ids[0] or False

        if field_id:
            field = field_pool.read(cr, uid, field_id)
            if field['ttype'] == 'selection':
                # if we have a fields.selection, we want to evaluate the 2nd part of the tuple which is user readable
                try:
                    dict_of_values = dict(self.pool.get(field['model'])._columns[line['name']].selection)
                except TypeError as e:
                    logging.getLogger('Track changes').warning("""Can\'t track changes for the field %s of the model %s. Error is %s"""
                                                            % (line['name'], model.name, e))

        # Get the values
        old_value = line.get('old_value')
        new_value = line.get('new_value')
        method = line.get('method')
        
#        if old_value == new_value and method not in ('create', 'unlink'):
#            continue
        # the check below is for the case where we have empty fields but with different types (i.e. transport_type that was comparing a unicode and a boolean)
        if not old_value:
            old_value = False
        if not new_value:
            new_value = False
        if new_value == old_value:
            continue  # nothing has changed, nothing to log
        
        # for the many2one field, we compare old_value and new_value with the name (uf_1624), so the 2nd part of the tupe (old_value[1] == new_value[1])
        if method not in ('create', 'unlink') and (old_value == new_value \
           or (field['ttype'] == 'datetime' and old_value and new_value and old_value[:10] == new_value[:10])\
           or (field['ttype'] == 'many2one' and old_value and new_value and old_value[1] == new_value[1])\
           or (field['ttype'] == 'selection' and old_value and new_value and dict_of_values.get(old_value) == dict_of_values.get(new_value))):
             continue

        res_id = line.get('res_id')
        name = line.get('name', '')
        object_id = line.get('object_id')
        user_id = line.get('user_id')
        timestamp = line.get('timestamp', time.strftime('%Y-%m-%d %H:%M:%S'))
        log = line.get('log')
        field_description = line.get('field_description', '')
        sub_obj_name = line.get('sub_obj_name', '')
        rule_id = line.get('rule_id')
        fct_res_id = line.get('fct_res_id')
        fct_object_id = line.get('fct_object_id')

        if res_id:
            # Get the log number
            seq_object_id = object_id
            seq_res_id = res_id
            fct_object = self.pool.get('ir.model').browse(cr, uid, seq_object_id)
            log_sequence = self.pool.get('audittrail.log.sequence').search(cr, uid, [('model', '=', fct_object.model), ('res_id', '=', seq_res_id)])
            if log_sequence:
                log_seq = self.pool.get('audittrail.log.sequence').browse(cr, uid, log_sequence[0]).sequence
                log = log_seq.get_id(code_or_id='id')
            else:
                # Create a new sequence
                seq_pool = self.pool.get('ir.sequence')
                seq_typ_pool = self.pool.get('ir.sequence.type')
                types = {
                    'name': fct_object.name,
                    'code': fct_object.model,
                }
                seq_typ_pool.create(cr, uid, types)
                seq = {
                    'name': fct_object.name,
                    'code': fct_object.model,
                    'prefix': '',
                    'padding': 1,
                }
                seq_id = seq_pool.create(cr, uid, seq)
                self.pool.get('audittrail.log.sequence').create(cr, uid, {'model': fct_object.model, 'res_id': seq_res_id, 'sequence': seq_id})
                log = self.pool.get('ir.sequence').browse(cr, uid, seq_id).get_id(code_or_id='id')


        if field_id:
            field_description = field['field_description']
            if field_description == 'Pricelist':
                field_description = 'Currency'

#            if field['ttype'] == 'many2one':
#                if type(old_value) == tuple:
#                    old_value = old_value[0]
#                if type(new_value) == tuple:
#                    new_value = new_value[0]


        vals = {
                "field_id": field_id,
                "old_value": old_value or '',
                "new_value": new_value or '',
                "field_description": field_description,
                "res_id": res_id,
                "name": name,
                "object_id": object_id,
                "user_id": user_id,
                "method": method,
                "timestamp": timestamp,
                "log": log,
                "rule_id": rule_id,
                "fct_res_id": fct_res_id,
                "fct_object_id": fct_object_id,
                "sub_obj_name": sub_obj_name,
                }
        log_line_pool.create(cr, uid, vals)
    #End Loop
    return True

def _get_domain_fields(self, domain=[]):
    '''
    Returns fields to read from the domain
    '''    
    ret_f = []
    for d in domain:
        ret_f.append(d[0])
       
    return ret_f

def _check_domain(self, cr, uid, vals=[], domain=[], model=False, res_id=False):
    '''
    Check if the values check with the domain
    '''
    res = True
    pool = pooler.get_pool(cr.dbname)
    for d in tuple(domain):
        assert d[1] in ('=', '!=', 'in', 'not in'), _("'%s' Not comprehensive operator... Please use only '=', '!=', 'in' and 'not in' operators") %(d[1])
            
        if len(d[0].split('.')) == 2 and model:
            p_rel, p_field = d[0].split('.')
            parent_field_id = pool.get('ir.model.fields').search(cr, uid, [('model', '=', model.model), ('name', '=', p_rel)])
            parent_field = pool.get('ir.model.fields').browse(cr, uid, parent_field_id)
            if not vals.get(p_rel) and res_id:
                vals[d[0]] = self.pool.get(model.model).read(cr, uid, res_id, [p_rel])[p_rel]
            if parent_field and parent_field[0].relation and vals.get(p_rel):
                if isinstance(vals[p_rel], (int, long)):
                    p_rel_id = vals[p_rel]
                else:
                    p_rel_id = vals[p_rel][0] 
                value = pool.get(parent_field[0].relation).read(cr, uid, p_rel_id, [p_field])
                if value:
                    d = (p_field, d[1], d[2])
                    vals[p_field] = value[p_field]
        
        if d[0] not in vals and model and res_id:
            obj = self.pool.get(model.model).read(cr, uid, res_id, [d[0]])
            vals[d[0]] = obj[d[0]]
        
        if d[1] == '=' and vals[d[0]] != d[2]:
            res = False
        elif d[1] == '!=' and vals[d[0]] == d[2]:
            res = False
        elif d[1] == 'in' and vals[d[0]] not in d[2]:
            res = False
        elif d[1] == 'not in' and vals[d[0]] in d[2]:
            res = False
            
    return res

def get_field_description(model):
    """
    Redefine the field_description for sale order and sale order line
    """
    if model.model == 'sale.order':
        field_description = 'Field Order'
    elif model.model == 'sale.order.line':
        field_description = 'Field Order Line'
    elif model.model== 'stock.picking':
        field_description = 'Incoming Shipment'
    else:
        field_description = model.name
    return field_description

def log_fct(self, cr, uid, model, method, fct_src, fields_to_trace=None, rule_id=False, parent_field_id=False, name_get_field='name', domain='[]', *args, **kwargs):
    """
    Logging function: This function is performs logging operations according to method
    @param cr: the current database
    @param uid: the current user’s ID for security checks,
    @param model: Object who's values are being changed
    @param method: method to log: create, read, write, unlink
    @param fct_src: execute method of Object proxy

    @return: Returns result as per method of Object proxy
    """
    if not fields_to_trace:
        fields_to_trace = []
    uid_orig = uid
    uid = 1
    pool = pooler.get_pool(cr.dbname)
    resource_pool = pool.get(model)
    model_pool = pool.get('ir.model')

    model_ids = model_pool.search(cr, uid, [('model', '=', model)])
    model_id = model_ids and model_ids[0] or False
    assert model_id, _("'%s' Model does not exist...") %(model,)
    model = model_pool.browse(cr, uid, model_id)
    domain = eval(domain)
    fields_to_read = ['id']

    old_values = {}
    if method in ('create'):
        res_id = fct_src(self, *args, **kwargs)
        
        # If the object doesn't match with the domain
        if domain and not _check_domain(self, cr, uid, args[2], domain, model, res_id):
            return res_id
        
        model_id = model.id
        model_name = model.name
        # If we are on the children object, escalate to the parent log
        if parent_field_id:
            parent_field = pool.get('ir.model.fields').browse(cr, uid, parent_field_id)
            model_id = model_pool.search(cr, uid, [('model', '=', parent_field.relation)])
            if not model_id or not args[2].get(parent_field.name, False):
                return res_id
            else:
                model_id = model_id[0]
            model_name = parent_field.model_id.name
            resource = resource_pool.read(cr, uid, res_id, [parent_field.name, name_get_field or 'name'])
            res_id2 = resource[parent_field.name][0]
        else:
            res_id2 = res_id
        
        vals = {
                "name": '%s' %model.name,
                "method": method,
                "object_id": model_id,
                "user_id": uid_orig,
                "res_id": res_id2,
                "field_description": get_field_description(model),
        }

        # Add the name of the created sub-object
        if parent_field_id:
            vals.update({'sub_obj_name': resource[name_get_field or 'name'],
                         'rule_id': rule_id,
                         'fct_object_id': model.id,
                         'fct_res_id': res_id})

        # We create only one line on creation (not one line by field)
        create_log_line(self, cr, uid, model, [vals])

        # Get new values
        if res_id and fields_to_trace:
            resource = resource_pool.read(cr, uid, res_id, fields_to_trace)
            if 'id' in resource:
                del resource['id']

            # now we create one line for each field tracked
            lines = []
            for field in resource.keys():
                line = vals.copy()
                line.update({
                      'name': field,
                      'new_value': resource[field],
                      })
                lines.append(line)

            create_log_line(self, cr, uid, model, lines)

        return res_id

    elif method in ('unlink'):
        res_ids = []
        if isinstance(args[2], (int, long)):
            res_ids = [args[2]]
        else:
            res_ids = list(args[2])
        model_name = model.name
        model_id = model.id
        fields_to_read = [name_get_field, 'name']
        fields_to_read.extend(_get_domain_fields(self, domain))
        
        if parent_field_id:
            parent_field = pool.get('ir.model.fields').browse(cr, uid, parent_field_id)
            model_id = model_pool.search(cr, uid, [('model', '=', parent_field.relation)])
            # If the parent object is not a valid object
            if not model_id:
                return fct_src(self, *args, **kwargs)
            else:
                model_id = model_id[0]
            model_name = parent_field.model_id.name
        
        for res_id in res_ids:
            old_values[res_id] = resource_pool.read(cr, uid, res_id, fields_to_read)
            # If the object doesn't match with the domain
            if domain and not _check_domain(self, cr, uid, old_values[res_id], domain, model, res_id):
                res_ids.pop(res_ids.index(res_id))
                continue
            if model_name == 'Sales Order':
                model_name = 'Field Order'
            elif model_name == 'Sales Order Line':
                model_name = 'Field Order Line'
            elif model_name == 'Picking List':
                model_name = 'Incoming Shipment'
            vals = {
                "name": "%s" %model_name,
                "method": method,
                "object_id": model_id,
                "user_id": uid_orig,
                "field_description": model_name,
            }

            if not parent_field_id:
                vals.update({'res_id': res_id})
            else:
                ressource = resource_pool.read(cr, uid, res_id, [parent_field.name, name_get_field or 'name'])
                res_id = ressource[parent_field.name]
                # Add the name of the created sub-object
                if res_id:
                    res_id = res_id[0]
                else:
                    continue
                vals = {
                        "name": "%s" %model_name,
                        "sub_obj_name": "%s" %ressource.get(name_get_field or 'name', ''),
                        "method": method,
                        "object_id": model_id,
                        "user_id": uid_orig,
                        "res_id": res_id,  
                        "field_description": model_name,
                        }

            # We create only one line when deleting a record
            create_log_line(self, cr, uid, model, [vals])
        res = fct_src(self, *args, **kwargs)
        return res
    else: 
        res_ids = []
        res = True
        fields = []
        if args:
            if isinstance(args[2], (long, int)):
                res_ids = [args[2]]
            else:
                res_ids = list(args[2])
            if len(args)>3 and type(args[3]) == dict:
                fields.extend(list(set(args[3]) & set(fields_to_trace)))
            # we take below the fields.function that were ignored
            fields_obj = self.pool.get('ir.model.fields')
            fields_to_trace_ids = fields_obj.search(cr, uid, [('name', 'in', fields_to_trace), ('model_id', '=', model_id)])
            for fields_value in fields_obj.read(cr, uid, fields_to_trace_ids, ['is_function', 'name']):
                if fields_value['is_function']:
                    fields.append(fields_value['name'])
                
        model_id = model.id

        # if no change on traced fields, variable fields is empty: so nothing to do.
        if fields:
            if parent_field_id:
                parent_field = pool.get('ir.model.fields').browse(cr, uid, parent_field_id)
                model_id = model_pool.search(cr, uid, [('model', '=', parent_field.relation)])
                # If the parent object is not a valid object
                if not model_id:
                    return res
                else:
                    model_id = model_id[0]
                    
                if parent_field.name not in fields:
                    fields.append(parent_field.name)
                
            fields.extend(_get_domain_fields(self, domain))
            # Remove double entries
            fields = list(set(fields))
            if name_get_field not in fields:
                fields.append(name_get_field)

            # Get old values
            if res_ids:
                for resource in resource_pool.read(cr, uid, res_ids, fields):
                    if parent_field_id and not args[3].get(parent_field.name, resource[parent_field.name]):
                        continue
                    if domain and not _check_domain(self, cr, uid, resource, domain, model):
                        res_ids.pop(res_ids.index(resource['id']))
                        continue
                    
                    resource_id = resource['id']
                    if 'id' in resource:
                        del resource['id']
                        
                    old_value = resource.copy()
#                for field in resource.keys():
#                    old_value = resource.copy()
                    
                    old_values[resource_id] = {'value': old_value}

        # Run the method on object
        res = fct_src(self, *args, **kwargs)

        # Get new values
        if fields and res_ids:
            for resource in resource_pool.read(cr, uid, res_ids, fields):
                if parent_field_id and not args[3].get(parent_field.name, resource[parent_field.name]):
                    continue
                res_id = resource['id']
                res_id2 = parent_field_id and resource[parent_field.name][0] or res_id
                if 'id' in resource:
                    del resource['id']

                vals = {
                    "method": method,
                    "object_id": model_id,
                    "user_id": uid_orig,
                    "res_id": res_id2,
                }
                if 'name' in resource:
                    vals.update({'name': resource['name']})

                # Add the name of the created sub-object
                if parent_field_id:
                    vals.update({'sub_obj_name': resource[name_get_field],
                                 'rule_id': rule_id,
                                 'fct_object_id': model.id,
                                 'fct_res_id': res_id})

                lines = []
                for field in resource.keys():
                    line = vals.copy()
                    line.update({
                          'name': field,
                          'new_value': resource[field],
                          'old_value': old_values[res_id]['value'][field],
                          })
                    lines.append(line)

                create_log_line(self, cr, uid, model, lines)
        return res
    return True


#########################################################################
#                                                                       #
# OVERRIDE OSV METHODS (only create, write and unlink for the moment)   #
#                                                                       #
#########################################################################

_old_create = orm.orm.create
_old_write = orm.orm.write
_old_unlink = orm.orm.unlink

def _audittrail_osv_method(self, old_method, method_name, cr, *args, **kwargs):
    """ General wrapper for osv methods """
    # If the object is not marked as traced object, just return the normal method
    if not self._trace:
        return old_method(self, *args, **kwargs)

    # If the object is traceable
    uid_orig = args[1]
    model = self._name
    pool = pooler.get_pool(cr.dbname)
    model_pool = pool.get('ir.model')
    rule_pool = pool.get('audittrail.rule')

    def my_fct(cr, uid, model, method, *args, **kwargs):
        rule = False
        model_ids = model_pool.search(cr, uid, [('model', '=', model)])
        model_id = model_ids and model_ids[0] or False

        if not model_id:
            return old_method(self, *args, **kwargs)

        if 'audittrail.rule' in pool.obj_list():
            rule = True

        if not rule:
            return old_method(self, *args, **kwargs)

        rule_ids = rule_pool.search(cr, uid, [('object_id', '=', model_id)])
        if not rule_ids:
            return old_method(self, *args, **kwargs)

        for thisrule in rule_pool.browse(cr, uid, rule_ids):
            # if the rule for the right method, then go inside and do the track change log
            if getattr(thisrule, 'log_' + method_name):
                fields_to_trace = [(field.name) for field in thisrule.field_ids]
                return log_fct(self, cr, uid_orig, model, method, old_method, fields_to_trace, thisrule.id, thisrule.parent_field_id.id, thisrule.name_get_field_id.name, thisrule.domain_filter, *args, **kwargs)
        
        return old_method(self, *args, **kwargs)
    res = my_fct(cr, uid_orig, model, method_name, *args, **kwargs)
    return res


def _audittrail_create(self, *args, **kwargs):
    """ Wrapper to trace the osv.create method """
    return _audittrail_osv_method(self, _old_create, 'create', args[0], *args, **kwargs)

def _audittrail_write(self, *args, **kwargs):
    """ Wrapper to trace the osv.write method """
    return _audittrail_osv_method(self, _old_write, 'write', args[0], *args, **kwargs)

def _audittrail_unlink(self, *args, **kwargs):
    """ Wrapper to trace the osv.unlink method """
    return _audittrail_osv_method(self, _old_unlink, 'unlink', args[0], *args, **kwargs)

orm.orm.create = _audittrail_create
orm.orm.write = _audittrail_write
orm.orm.unlink = _audittrail_unlink

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
